<?php

namespace Drupal\suspect_blocker\Controller;

use Drupal\ban\BanIpManagerInterface;
use Drupal\Core\Controller\ControllerBase;
use Drupal\Core\Database\Connection;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Link;
use Drupal\Core\Url;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Controller for the Suspect Blocker report page.
 */
class SuspectBlockerReportController extends ControllerBase {

  /**
   * The database connection.
   *
   * @var \Drupal\Core\Database\Connection
   */
  protected Connection $database;

  /**
   * The module handler.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected $moduleHandler;

  /**
   * The ban IP manager service.
   *
   * @var BanIpManagerInterface
   */
  protected BanIpManagerInterface $banIpManager;

  /**
   * Constructs a new SuspectBlockerReportController object.
   *
   * @param \Drupal\Core\Database\Connection $database
   *   The database connection service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   The module handler service.
   * @param BanIpManagerInterface $banIpManager
   *   The ban IP manager service.
   */
  public function __construct(Connection $database, ModuleHandlerInterface $moduleHandler, BanIpManagerInterface $banIpManager) {
    $this->database = $database;
    $this->moduleHandler = $moduleHandler;
    $this->banIpManager = $banIpManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('database'),
      $container->get('module_handler'),
      $container->get('ban.ip_manager')
    );
  }

  /**
   * Removes a blocked IP and unbans it if needed.
   *
   * @param string $ip
   *   The IP address to be deleted.
   *
   * @return \Symfony\Component\HttpFoundation\RedirectResponse
   *   Redirects to the report page.
   */
  public function deleteIp(string $ip): RedirectResponse
  {
    // Remove from blocked IPs.
    $this->database->delete('suspect_blocker_blocked_ips')
      ->condition('ip_address', $ip)
      ->execute();

    // Remove from logs.
    $this->database->delete('suspect_blocker_logs')
      ->condition('ip_address', $ip)
      ->execute();

    // If Ban module is enabled, unban the IP.
    if ($this->moduleHandler->moduleExists('ban')) {
      $this->banIpManager->unbanIp($ip);
    }

    return $this->redirect('suspect_blocker.report');
  }

  /**
   * Generates the report page with a list of blocked IPs.
   *
   * @return array
   *   A render array with the table.
   */
  public function reportPage(): array
  {
    $query = $this->database->select('suspect_blocker_blocked_ips', 'b')
      ->fields('b', ['ip_address', 'blocked_time'])
      ->orderBy('blocked_time', 'DESC')
      ->execute();

    $rows = [];
    foreach ($query as $record) {
      $delete_url = Url::fromRoute('suspect_blocker.delete_ip', ['ip' => $record->ip_address]);
      $delete_link = Link::fromTextAndUrl($this->t('Delete'), $delete_url)->toString();
      $rows[] = [
        $record->ip_address,
        $record->blocked_time,
        $delete_link,
      ];
    }

    $header = ['IP Address', 'Blocked Time', 'Actions'];
    return [
      '#type' => 'table',
      '#header' => $header,
      '#rows' => $rows,
      '#empty' => $this->t('No blocked IPs found.'),
    ];
  }

}