<?php

namespace Drupal\suspect_blocker\EventSubscriber;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\suspect_blocker\Service\BotDetector;
use Drupal\suspect_blocker\Service\BlockerService;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Symfony\Component\EventDispatcher\EventSubscriberInterface;
use Symfony\Component\HttpFoundation\RequestStack;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\Event\RequestEvent;
use Symfony\Component\HttpKernel\Exception\HttpExceptionInterface;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Subscribes to kernel events to detect and block suspicious IPs.
 */
class SuspectBlockerSubscriber implements EventSubscriberInterface {

  /**
   * The request stack service.
   *
   * @var RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * The configuration object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * The current user.
   *
   * @var \Drupal\Core\Session\AccountInterface
   */
  protected $currentUser;

  /**
   * The bot Detector Service.
   *
   * @var \Drupal\suspect_blocker\Service\BotDetector
   */
  protected $botDetector;

  /**
   * The Blocker Service.
   *
   * @var \Drupal\suspect_blocker\Service\BlockerService
   */
  protected $blockerService;

  /**
   * Constructs a new SuspectBlockerSubscriber.
   *
   * @param RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   The config factory.
   * @param \Drupal\Core\Session\AccountInterface $current_user
   *   The current user.
   * @param Drupal\suspect_blocker\Service\BotDetector $botDetector
   *   The bot detector.
   * @param Drupal\suspect_blocker\Service\BlockerService $blockerService
   *   The Blocker Service.
   */
  public function __construct(
    RequestStack $request_stack,
    ConfigFactoryInterface $configFactory,
    AccountInterface $current_user,
    BotDetector $botDetector,
    BlockerService $blockerService
  ) {
    $this->requestStack = $request_stack;
    $this->configFactory = $configFactory;
    $this->currentUser = $current_user;
    $this->botDetector = $botDetector;
    $this->blockerService = $blockerService;
  }

  /**
   * {@inheritdoc}
   */
  public static function getSubscribedEvents(): array
  {
    return [
      KernelEvents::REQUEST => ['onKernelRequest', 100],
      KernelEvents::EXCEPTION => 'onKernelException',
    ];
  }

  public function onKernelRequest(RequestEvent $event): void
  {    
    if (!$event->isMainRequest()) {
      return;
    }

    $config = $this->configFactory->get('suspect_blocker.settings');
    if (!$config->get('bot_detector') || $this->shouldBypassRequest($config)) {
      return;
    }

    $request = $event->getRequest();
    $ip = $request->getClientIp();

    if ($this->botDetector->isSuspiciousBotRequest()) {
      $this->blockerService->banIp($ip);
    }

    $this->blockerService->logRequest($ip, $request->getPathInfo(), 200);
    $this->blockerService->handleIfAbuseDetected($ip);
  }
  
  /**
   * Handles kernel exception events to detect suspicious requests.
   *
   * @param \Symfony\Component\HttpKernel\Event\ExceptionEvent $event
   *   The exception event.
   */
  public function onKernelException(ExceptionEvent $event): void {
    $config = $this->configFactory->get('suspect_blocker.settings');
    if ($this->shouldBypassRequest($config)) {
      return;
    }

    $exception = $event->getThrowable();
    $request = $event->getRequest();
    $ip = $request->getClientIp();
    $path = $request->getPathInfo();

    if ($exception instanceof HttpExceptionInterface) {
      $status_code = $exception->getStatusCode();
      $monitored_codes = array_filter($config->get('monitored_status_codes') ?: [403, 404]);

      if (in_array($status_code, $monitored_codes)) {
        $this->blockerService->logRequest($ip, $path, (int) $status_code);
        $this->blockerService->handleIfAbuseDetected($ip);
      }
    }
  }

  /**
   * Checks Bypass on Requests.
   *
   * @return bool
   *   TRUE if Dev mode is enabled OR Authenticated User; FALSE otherwise.
   */
  protected function shouldBypassRequest($config): bool {
    return $config->get('develop_mode') || $this->currentUser->isAuthenticated();
  }

}
