<?php

namespace Drupal\suspect_blocker\Form;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Logger\LoggerChannelFactoryInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Configuration form for Suspect Blocker module.
 *
 * Allows administrators to configure settings and purge suspect blocker tables.
 */
class SuspectBlockerSettingsForm extends ConfigFormBase {

  /**
   * The Caches log.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface;
   */
  protected CacheBackendInterface $cacheLogs;
  protected CacheBackendInterface $cacheBlockedIps;
  
  /**
   * Logger channel for suspect_blocker.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected $logger;

  /**
   * Class constructor with dependency injection.
   *
   * @param \DDrupal\Core\Cache\CacheBackendInterface;
   *   Cache service.
   * @param \Drupal\Core\Logger\LoggerChannelFactoryInterface $logger_factory
   *   Logger channel factory service.
   */
  public function __construct(CacheBackendInterface $cacheLogs, CacheBackendInterface $cacheBlockedIps, LoggerChannelFactoryInterface $logger_factory) {
    $this->cacheLogs = $cacheLogs;
    $this->cacheBlockedIps = $cacheBlockedIps;
    $this->logger = $logger_factory->get('suspect_blocker');
  }

  /**
   * {@inheritdoc}
   *
   * Creates an instance of this form with services from the container.
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('cache.suspect_blocker_logs'),
      $container->get('cache.suspect_blocker_blocked_ips'),
      $container->get('logger.factory')
    );
  }

  /**
   * {@inheritdoc}
   *
   * Returns the names of configuration objects editable by this form.
   */
  protected function getEditableConfigNames() {
    return ['suspect_blocker.settings'];
  }

  /**
   * {@inheritdoc}
   *
   * Returns the unique ID of the form.
   */
  public function getFormId() {
    return 'suspect_blocker_settings';
  }

  /**
   * {@inheritdoc}
   *
   * Builds the configuration form.
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $config = $this->config('suspect_blocker.settings');
    
    // Number field to set ban threshold.
    $form['ban_threshold'] = [
      '#type' => 'number',
      '#title' => $this->t('Ban Threshold'),
      '#description' => $this->t('The number of suspicious requests within the time window that triggers banning.'),
      '#default_value' => $config->get('ban_threshold') ?: 5,
      '#min' => 1,
      '#required' => TRUE,
    ];

    // Number field to set time window in seconds.
    $form['time_window'] = [
      '#type' => 'number',
      '#title' => $this->t('Time Window (Seconds)'),
      '#description' => $this->t('The time window in seconds (burst threshold) to detect rapid requests.'),
      '#default_value' => $config->get('time_window') ?: 60,
      '#min' => 1,
      '#required' => TRUE,
    ];

    // Define the possible status codes to monitor.
    $available_status_codes = [
      200 => $this->t('200 Success'),
      400 => $this->t('400 Bad Request'),
      401 => $this->t('401 Unauthorized'),
      403 => $this->t('403 Forbidden'),
      404 => $this->t('404 Not Found'),
      429 => $this->t('429 Too Many Requests'),
    ];

    $selected_codes = $config->get('monitored_status_codes') ?: [403, 404];
    $form['monitored_status_codes'] = [
      '#type' => 'checkboxes',
      '#title' => $this->t('Monitored HTTP Status Codes'),
      '#description' => $this->t('Select the HTTP status codes to monitor and handle as suspicious requests.'),
      '#options' => $available_status_codes,
      '#default_value' => $selected_codes,
    ];

    $form['develop_mode'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Development Mode'),
      '#default_value' => $config->get('develop_mode') ?: FALSE,
      '#description' => $this->t('<strong>Warning:</strong> Do not enable this in production! This bypasses all blocking rules for testing purposes.'),
    ];

    $form['bot_detector'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Enable Bot Detector'),
      '#default_value' => $config->get('bot_detector') ?: FALSE,
      '#description' => $this->t('Enable detection mechanisms to identify and mitigate automated bot traffic.'),
    ];

    $form['risk_level'] = [
      '#type' => 'select',
      '#title' => $this->t('Risk Level'),
      '#options' => [
        6 => $this->t('Lowest'),
        5 => $this->t('Low'),
        4 => $this->t('Medium'),
        3 => $this->t('High'),
        2 => $this->t('Highest'),
      ],
      '#default_value' => $config->get('risk_level') ?: 4,
      '#description' => $this->t('Select the threat level associated with this action.'),
      '#states' => [
        'enabled' => [
          ':input[name="bot_detector"]' => ['checked' => TRUE],
        ],
      ],
    ];

    // Submit button to purge suspect blocker tables.
    $form['purge_tables'] = [
      '#type' => 'submit',
      '#value' => $this->t('Purge Suspect Blocker Caches'),
      // Limit validation errors to none to avoid validating other fields on click.
      '#limit_validation_errors' => [],
      // Specify the custom submit handler for this button.
      '#submit' => ['::purgeTables'],
      '#suffix' => '<p class="resize-both reset-appearance"><small class="form-item__description">' . $this->t('This does NOT purge the banned IPs, only the suspect_blocker tables.') . '</small></p>',
    ];

    return parent::buildForm($form, $form_state);
  }

  /**
   * {@inheritdoc}
   *
   * Handles submission of configuration values.
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    // Save config values from the form.
    $this->config('suspect_blocker.settings')
      ->set('ban_threshold', $form_state->getValue('ban_threshold'))
      ->set('time_window', $form_state->getValue('time_window'))
      ->set('monitored_status_codes', $form_state->getValue('monitored_status_codes'))
      ->set('develop_mode', $form_state->getValue('develop_mode'))
      ->set('bot_detector', $form_state->getValue('bot_detector'))
      ->set('risk_level', $form_state->getValue('risk_level'))
      ->save();

    parent::submitForm($form, $form_state);
  }

  /**
   * Custom submit handler to purge suspect blocker caches.
   *
   * Deletes all records from suspect_blocker_logs and suspect_blocker_blocked_ips caches.
   * Adds status or error messages to the backend and logs the purge action.
   *
   * @param array $form
   *   The form array.
   * @param \Drupal\Core\Form\FormStateInterface $form_state
   *   The form state object.
   * 
   * @throws Exception
   */
  public function purgeTables(array &$form, FormStateInterface $form_state): void {
    try {
      $this->cacheLogs->deleteAll();
      $this->cacheBlockedIps->deleteAll();
  
      $this->messenger()->addStatus($this->t('Suspect Blocker cache has been cleared.'));
      $this->logger->notice('Suspect Blocker cache has been purged by an administrator.');
    }
    catch (\Exception $e) {
      $this->messenger()->addError($this->t('Failed to purge cache: @msg', ['@msg' => $e->getMessage()]));
    }
  
    $form_state->setRedirect('suspect_blocker.settings');
  }

}
