<?php

namespace Drupal\suspect_blocker\Service;

use Drupal\ban\BanIpManagerInterface;
use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Psr\Log\LoggerInterface;

/**
 * Service for detecting suspicious IP requests and banning IPs based on thresholds.
 */
class BlockerService {

  /**
   * Cache backend service.
   *
   * @var \Drupal\Core\Cache\CacheBackendInterface
   */
  protected CacheBackendInterface $cacheLogs;
  protected CacheBackendInterface $cacheBlockedIps;

  /**
   * Logger channel.
   *
   * @var \Psr\Log\LoggerInterface
   */
  protected LoggerInterface $logger;

  /**
   * Ban IP manager service.
   *
   * @var \Drupal\ban\BanIpManagerInterface
   */
  protected BanIpManagerInterface $banIpManager;

  /**
   * Module handler service.
   *
   * @var \Drupal\Core\Extension\ModuleHandlerInterface
   */
  protected ModuleHandlerInterface $moduleHandler;

  /**
   * Array of HTTP status codes to monitor.
   *
   * @var int[]
   */
  protected array $monitoredStatusCodes;

  /**
   * Number of suspicious requests from an IP to trigger a ban.
   *
   * @var int
   */
  protected int $banThreshold;

  /**
   * Time window in seconds to count suspicious requests.
   *
   * @var int
   */
  protected int $timeWindow;

  /**
   * Constructs a BlockerService object.
   *
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheLogs
   *   Cache backend service.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cacheBlockedIps
   *   Cache backend service.
   * @param \Psr\Log\LoggerInterface $logger
   *   Logger channel for suspect_blocker.
   * @param \Drupal\ban\BanIpManagerInterface $banIpManager
   *   Ban IP manager service.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $moduleHandler
   *   Module handler service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $configFactory
   *   Configuration factory to get module settings.
   */
  public function __construct(
    CacheBackendInterface $cacheLogs,
    CacheBackendInterface $cacheBlockedIps,
    LoggerInterface $logger,
    BanIpManagerInterface $banIpManager,
    ModuleHandlerInterface $moduleHandler,
    ConfigFactoryInterface $configFactory
  ) {
    $this->cacheLogs = $cacheLogs;
    $this->cacheBlockedIps = $cacheBlockedIps;
    $this->logger = $logger;
    $this->banIpManager = $banIpManager;
    $this->moduleHandler = $moduleHandler;

    // Load module settings.
    $config = $configFactory->get('suspect_blocker.settings');
    $this->monitoredStatusCodes = array_filter($config->get('monitored_status_codes') ?? [403, 404]);
    $this->banThreshold = (int) ($config->get('ban_threshold') ?? 5);
    $this->timeWindow = (int) ($config->get('time_window') ?? 60);
  }

  /**
   * Logs a suspicious request by incrementing the count in cache.
   *
   * @param string $ip
   *   IP address of the requester.
   * @param string $path
   *   The requested URL path (not used in cache logging).
   * @param int $statusCode
   *   HTTP status code returned.
   */
  public function logRequest(string $ip, string $path, int $statusCode): void {
    if (!in_array($statusCode, $this->monitoredStatusCodes, TRUE)) {
      return;
    }

    $cache_id = $this->getCacheId($ip);
    $cache = $this->cacheLogs->get($cache_id);
    $current_time = time();

    if ($cache) {
      $data = $cache->data;
      if (($current_time - $data['first_timestamp']) > $this->timeWindow) {
        $data = [
          'count' => 1,
          'first_timestamp' => $current_time,
        ];
      }
      else {
        $data['count']++;
      }
    }
    else {
      $data = [
        'count' => 1,
        'first_timestamp' => $current_time,
      ];
    }

    $this->cacheLogs->set($cache_id, $data, $current_time + $this->timeWindow);
  }

  /**
   * Checks if the IP has exceeded the threshold for suspicious requests.
   *
   * @param string $ip
   *   IP address to check.
   *
   * @return bool
   *   TRUE if abuse detected, FALSE otherwise.
   */
  public function isAbuseDetected(string $ip): bool {
    $cache_id = $this->getCacheId($ip);
    $cache = $this->cacheLogs->get($cache_id);

    if (!$cache) {
      return FALSE;
    }

    return ($cache->data['count'] >= $this->banThreshold);
  }

  /**
   * Handles banning the IP if abuse is detected.
   *
   * @param string $ip
   *   IP address to check and ban.
   */
  public function handleIfAbuseDetected(string $ip): void {
    if (!$this->isAbuseDetected($ip)) {
      return;
    }

    // If ban module not enabled, just log and return.
    if (!$this->moduleHandler->moduleExists('ban')) {
      $this->logger->warning("Ban module not enabled, IP $ip detected as suspicious but not banned.");
      return;
    }

    try {
      $this->banIpManager->banIp($ip);
      $this->logger->warning("IP $ip banned for suspicious activity.");

      // Clear cache for this IP after banning.
      $this->cacheLogs->delete($this->getCacheId($ip));
      $this->cacheBlockedIps->set('blocked:' . $ip . 'at {time}.', ['time' => time()]);
    }
    catch (\Exception $e) {
      $this->logger->error("Failed to ban IP $ip: " . $e->getMessage());
    }
  }

  /**
   * Returns a cache ID for storing IP-related data.
   *
   * @param string $ip
   *   IP address.
   *
   * @return string
   *   Cache ID.
   */
  protected function getCacheId(string $ip): string {
    return 'suspect_blocker:' . $ip;
  }

}
