<?php

namespace Drupal\suspect_blocker\Service;

use Drupal\Core\Config\ConfigFactoryInterface;
use Symfony\Component\HttpFoundation\RequestStack;

/**
 * Service to detect suspicious bot-like requests.
 */
class BotDetector {

  /**
   * The request stack service.
   *
   * @var RequestStack
   */
  protected RequestStack $requestStack;

  /**
   * The configuration object.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected ConfigFactoryInterface $configFactory;

  /**
   * Constructs a new SuspectBlockerSubscriber.
   *
   * @param RequestStack $request_stack
   *   The request stack.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   */
  public function __construct(RequestStack $request_stack, ConfigFactoryInterface $config_factory) {
    $this->requestStack = $request_stack;
    $this->configFactory = $config_factory;
  }

  /**
   * Determines if the current request is likely from a suspicious bot.
   *
   * @return bool True if the request is suspicious, false otherwise.
   */
  public function isSuspiciousBotRequest(): bool {
    $config = $this->configFactory->get('suspect_blocker.settings');
    $riskLevel = (int) ($config->get('risk_level') ?? 4);

    $request = $this->requestStack->getCurrentRequest();
    if (!$request) {
      // No request, can't evaluate.
      return false;
    }

    $headers = $request->headers;
    $ua = $headers->get('User-Agent') ?? '';

    $score = 0;

    // Check User-Agent suspiciousness (2 points).
    if (empty($ua) || preg_match('/bot|crawler|spider|curl|wget|python|scrapy|httpclient|http_request2/i', $ua)) {
      $score += 2;
    }

    // Missing headers (1 point each).
    foreach (['Accept-Language', 'Referer'] as $header) {
      if (!$headers->has($header)) {
        $score++;
      }
    }

    // Accept header */* (1 point).
    if ($headers->has('Accept') && preg_match('/\*\/\*/', $headers->get('Accept'))) {
      $score++;
    }

    // Connection header 'close' or missing (1 point).
    $connection = $headers->get('Connection');
    if ($connection === 'close' || $connection === null) {
      $score++;
    }

    // GET with JSON or urlencoded Content-Type (1 point).
    if (
      $request->getMethod() === 'GET' &&
      preg_match('/json|urlencoded/i', (string) $headers->get('Content-Type'))
    ) {
      $score++;
    }

    return $score > $riskLevel;
  }

}
