<?php

namespace Drupal\Tests\suspect_blocker\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Drupal\suspect_blocker\Controller\SuspectBlockerReportController;
use Symfony\Component\HttpFoundation\RedirectResponse;

/**
 * Tests for the SuspectBlockerReportController using real DB data.
 *
 * @group suspect_blocker
 */
class SuspectBlockerReportControllerKernelTest extends KernelTestBase {

  /**
   * Modules to enable.
   *
   * Adding 'ban' ensures the ban.ip_manager service is available.
   */
  protected static $modules = [
    'system',
    'ban',
    'suspect_blocker',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    // Install the custom table schemas.
    $this->installSchema('suspect_blocker', [
      'suspect_blocker_blocked_ips',
      'suspect_blocker_logs',
    ]);
    // Install the schema for the Ban module.
    $this->installSchema('ban', ['ban_ip']);
  }

  /**
   * Test that deleteIp removes records from the database.
   */
  public function testDeleteIpRemovesData() {
    $ip = '1.2.3.4';
    $database = $this->container->get('database');

    // Insert a record into suspect_blocker_blocked_ips.
    $database->insert('suspect_blocker_blocked_ips')
      ->fields([
        'ip_address' => $ip,
        'blocked_time' => date('Y-m-d H:i:s'),
      ])
      ->execute();

    // Insert a record into suspect_blocker_logs.
    $database->insert('suspect_blocker_logs')
      ->fields([
        'ip_address' => $ip,
        'path' => '/test',
        'status_code' => 404,
        'timestamp' => time(),
      ])
      ->execute();

    // Instantiate the controller using the factory method.
    $controller = SuspectBlockerReportController::create($this->container);

    // Call deleteIp.
    $response = $controller->deleteIp($ip);
    $this->assertInstanceOf(RedirectResponse::class, $response);

    // Verify that the records are removed.
    $blocked = $database->select('suspect_blocker_blocked_ips', 'b')
      ->fields('b')
      ->condition('ip_address', $ip)
      ->execute()
      ->fetchAll();
    $logs = $database->select('suspect_blocker_logs', 'l')
      ->fields('l')
      ->condition('ip_address', $ip)
      ->execute()
      ->fetchAll();

    $this->assertEmpty($blocked, 'Blocked IP record was removed.');
    $this->assertEmpty($logs, 'Log records were removed.');
  }

  /**
   * Test that reportPage returns a render array with inserted data.
   */
  public function testReportPageDisplaysData() {
    $database = $this->container->get('database');

    // Clear any existing data.
    $database->truncate('suspect_blocker_blocked_ips')->execute();

    // Insert test records.
    $database->insert('suspect_blocker_blocked_ips')
      ->fields([
        'ip_address' => '1.2.3.4',
        'blocked_time' => '2025-02-11 10:00:00',
      ])
      ->execute();
    $database->insert('suspect_blocker_blocked_ips')
      ->fields([
        'ip_address' => '5.6.7.8',
        'blocked_time' => '2025-02-11 09:00:00',
      ])
      ->execute();

    // Instantiate the controller via create().
    $controller = SuspectBlockerReportController::create($this->container);

    $build = $controller->reportPage();
    $this->assertEquals('table', $build['#type']);
    $this->assertCount(2, $build['#rows']);
  }
}