<?php

namespace Drupal\Tests\suspect_blocker\Kernel;

use Drupal\KernelTests\KernelTestBase;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpKernel\Event\ExceptionEvent;
use Symfony\Component\HttpKernel\Exception\HttpException;
use Symfony\Component\HttpKernel\HttpKernelInterface;
use Symfony\Component\HttpKernel\KernelEvents;

/**
 * Tests for the SuspectBlockerSubscriber without mocks.
 *
 * @group suspect_blocker
 */
class SuspectBlockerSubscriberKernelTest extends KernelTestBase {

  /**
   * Modules to enable.
   *
   * We include 'system', 'ban', and our suspect_blocker module.
   */
  protected static $modules = [
    'system',
    'ban',
    'suspect_blocker',
  ];

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();
    // Install custom table schemas for suspect_blocker.
    $this->installSchema('suspect_blocker', [
      'suspect_blocker_logs',
      'suspect_blocker_blocked_ips',
    ]);
    // Install Ban module's schema.
    $this->installSchema('ban', ['ban_ip']);

    // Configure suspect_blocker settings so that a single suspicious request triggers a ban.
    $this->container->get('config.factory')->getEditable('suspect_blocker.settings')
      ->set('ban_threshold', 1)
      ->set('time_window', 60)
      ->save();
  }

  /**
   * Test that the subscriber bans an IP after a suspicious request.
   */
  public function testOnKernelExceptionBansIp() {
    // Create a request with a specific client IP.
    $request = Request::create('/nonexistent');
    $request->server->set('REMOTE_ADDR', '1.2.3.4');

    // Create a 404 HTTP exception.
    $exception = new HttpException(404, 'Not Found');

    // Build the ExceptionEvent.
    $kernel = $this->container->get('http_kernel');
    $event = new ExceptionEvent($kernel, $request, HttpKernelInterface::MAIN_REQUEST, $exception);

    // Dispatch the exception event.
    $this->container->get('event_dispatcher')->dispatch($event, KernelEvents::EXCEPTION);

    // Get the database service.
    $database = $this->container->get('database');

    // Verify that the IP is banned by checking suspect_blocker_blocked_ips.
    $record = $database->select('suspect_blocker_blocked_ips', 'b')
      ->fields('b')
      ->condition('ip_address', '1.2.3.4')
      ->execute()
      ->fetchAssoc();
    $this->assertNotEmpty($record, 'The IP 1.2.3.4 is banned in suspect_blocker_blocked_ips.');

    // Verify that logs for that IP have been cleared.
    $logs = $database->select('suspect_blocker_logs', 'l')
      ->fields('l')
      ->condition('ip_address', '1.2.3.4')
      ->execute()
      ->fetchAll();
    $this->assertEmpty($logs, 'Logs for the banned IP have been cleared.');
  }
}